@@ -5,7 +5,6 @@ use warnings;
 no warnings 'numeric';
 no warnings 'uninitialized';
 no warnings 'recursion';
-use feature ':5.10';
 
 use Yote::Array;
 use Yote::Hash;
@@ -28,7 +27,7 @@ our $FIRST_ID;
 
 use vars qw($VERSION);
 
-$VERSION = '0.06';
+$VERSION = '0.07';
 
 
 # ------------------------------------------------------------------------------------------
@@ -120,59 +119,52 @@ sub fetch {
     # Return the object if we have a reference to its dirty state.
     #
     my $ref = $Yote::ObjProvider::DIRTY->{$id} || $Yote::ObjProvider::WEAK_REFS->{$id} || $CACHE->fetch( $id );
-
-    if( $ref && $LOCKER && $id ne first_id() && (! $LOCKER->locked_by_me( $id ) ) ) {
-	my $ldt = $LOCKER->check_last_dirty_time( $id );
-#	print STDERR "[$$ ".time()."] GOT A REF for $id at $ldt. its last load time was $Yote::ObjProvider::LAST_LOAD_TIME->{$id}\n";
-	if( $ldt && $Yote::ObjProvider::LAST_LOAD_TIME->{$id} <= $ldt ) {
-#	    print STDERR "[$$ ".time()."] FLUSHING $id \n";
-	    flush( $id );
-	    $ref = undef;
-	}
-    }
-
+#	print STDERR "[$$ ".time()."] cached $ref $id, checking on LOCKER\n";
     if( $LOCKER ) { 
-	$LOCKER->lock_object( $id );
+	$ref = $LOCKER->lock_object( $id, $ref );
     }
 
-    return $ref if $ref;
-
+    if( $ref ) {
+#	print STDERR "[$$ ".time()."] returning ref $ref for $id\n";
+	return $ref;
+    }
+#    print STDERR "[$$ ".time()."] LOADING $id\n";
     my $obj_arry = $DATASTORE->fetch( $id );
     $Yote::ObjProvider::LAST_LOAD_TIME->{$id} = time();
+
 #    print STDERR "[$$ ".time()."] OBJ $id loaded at $Yote::ObjProvider::LAST_LOAD_TIME->{$id}\n";
 
     if( $obj_arry ) {
         my( $id, $class, $data ) = @$obj_arry;
-        given( $class ) {
-            when('ARRAY') {
-                my( @arry );
-                tie @arry, 'Yote::Array', $id, @$data;
-                my $tied = tied @arry; $tied->[2] = \@arry;
-                __store_weak( $id, \@arry );
-		$CACHE->stow( $id, \@arry );
-                return \@arry;
-            }
-            when('HASH') {
-                my( %hash );
-                tie %hash, 'Yote::Hash', $id, map { $_ => $data->{$_} } keys %$data;
-                my $tied = tied %hash; $tied->[2] = \%hash;
-                __store_weak( $id, \%hash );
-		$CACHE->stow( $id, \%hash );
-                return \%hash;
-            }
-            default {
-                eval("require $class");
-		print STDERR Data::Dumper->Dump([$class,$!,$@]) if $@;
-                my $obj = $class->new( $id );
-                $obj->{DATA} = $data;
-                $obj->{ID} = $id;
-		$obj->_load();
-                __store_weak( $id, $obj );
-		$CACHE->stow( $id, $obj );
-                return $obj;
-            }
-        }
+	if( $class eq 'ARRAY' ) {
+	    my( @arry );
+	    tie @arry, 'Yote::Array', $id, @$data;
+	    my $tied = tied @arry; $tied->[2] = \@arry;
+	    __store_weak( $id, \@arry );
+	    $CACHE->stow( $id, \@arry );
+	    return \@arry;
+	}
+	elsif( $class eq 'HASH' ) {
+	    my( %hash );
+	    tie %hash, 'Yote::Hash', $id, map { $_ => $data->{$_} } keys %$data;
+	    my $tied = tied %hash; $tied->[2] = \%hash;
+	    __store_weak( $id, \%hash );
+	    $CACHE->stow( $id, \%hash );
+	    return \%hash;
+	}
+	else {
+	    eval("require $class");
+	    print STDERR Data::Dumper->Dump([$class,$!,$@]) if $@;
+	    my $obj = $class->new( $id );
+	    $obj->{DATA} = $data;
+	    $obj->{ID} = $id;
+	    $obj->_load();
+	    __store_weak( $id, $obj );
+	    $CACHE->stow( $id, $obj );
+	    return $obj;
+	}
     }
+
     return undef;
 } #fetch
 
@@ -180,59 +172,58 @@ sub fetch {
 sub get_id {
     my $ref = shift;
     my $class = ref( $ref );
-    given( $class ) {
-        when('Yote::Array') {
-            return $ref->[0];
-        }
-        when('ARRAY') {
-            my $tied = tied @$ref;
-            if( $tied ) {
-                $tied->[0] ||= $DATASTORE->get_id( "ARRAY" );
-                __store_weak( $tied->[0], $ref );
-                return $tied->[0];
-            }
-            my( @data ) = @$ref;
-            my $id = $DATASTORE->get_id( $class );
-            tie @$ref, 'Yote::Array', $id;
-	    $tied = tied @$ref; $tied->[2] = $ref;
-            push( @$ref, @data );
-            dirty( $ref, $id );
-            __store_weak( $id, $ref );
-            return $id;
-        }
-        when('Yote::Hash') {
-            my $wref = $ref;
-            return $ref->[0];
-        }
-        when('HASH') {
-            my $tied = tied %$ref;
-
-            if( $tied ) {
-                $tied->[0] ||= $DATASTORE->get_id( "HASH" );
-                __store_weak( $tied->[0], $ref );
-                return $tied->[0];
-            }
-            my $id = $DATASTORE->get_id( $class );
-            my( %vals ) = %$ref;
-            tie %$ref, 'Yote::Hash', $id;
-	    $tied = tied %$ref; $tied->[2] = $ref;
-            for my $key (keys %vals) {
-                $ref->{$key} = $vals{$key};
-            }
-            dirty( $ref, $id );
-            __store_weak( $id, $ref );
-            return $id;
-        }
-        default {
-	    if( $class eq 'Yote::YoteRoot' ) {
-		$ref->{ID} = $DATASTORE->first_id( $class );
-	    } else {
-		$ref->{ID} ||= $DATASTORE->get_id( $class );
-	    }
-            __store_weak( $ref->{ID}, $ref );
-            return $ref->{ID};
-        }
+    if( $class eq 'Yote::Array') {
+	return $ref->[0];
     }
+    elsif( $class eq 'ARRAY' ) {
+	my $tied = tied @$ref;
+	if( $tied ) {
+	    $tied->[0] ||= $DATASTORE->get_id( "ARRAY" );
+	    __store_weak( $tied->[0], $ref );
+	    return $tied->[0];
+	}
+	my( @data ) = @$ref;
+	my $id = $DATASTORE->get_id( $class );
+	tie @$ref, 'Yote::Array', $id;
+	$tied = tied @$ref; $tied->[2] = $ref;
+	push( @$ref, @data );
+	dirty( $ref, $id );
+	__store_weak( $id, $ref );
+	return $id;
+    }
+    elsif( $class eq 'Yote::Hash' ) {
+	my $wref = $ref;
+	return $ref->[0];
+    }
+    elsif( $class eq 'HASH' ) {
+	my $tied = tied %$ref;
+
+	if( $tied ) {
+	    $tied->[0] ||= $DATASTORE->get_id( "HASH" );
+	    __store_weak( $tied->[0], $ref );
+	    return $tied->[0];
+	}
+	my $id = $DATASTORE->get_id( $class );
+	my( %vals ) = %$ref;
+	tie %$ref, 'Yote::Hash', $id;
+	$tied = tied %$ref; $tied->[2] = $ref;
+	for my $key (keys %vals) {
+	    $ref->{$key} = $vals{$key};
+	}
+	dirty( $ref, $id );
+	__store_weak( $id, $ref );
+	return $id;
+    }
+    else {
+	if( $class eq 'Yote::YoteRoot' ) {
+	    $ref->{ID} = $DATASTORE->first_id( $class );
+	} else {
+	    $ref->{ID} ||= $DATASTORE->get_id( $class );
+	}
+	__store_weak( $ref->{ID}, $ref );
+	return $ref->{ID};
+    }
+
 } #get_id
 
 sub list_insert {
@@ -379,50 +370,47 @@ sub stow {
     die unless $id;
 
     my $data = __raw_data( $obj );
-    given( $class ) {
-        when('ARRAY') {
-            $DATASTORE->stow( $id,'ARRAY', $data );
-            __clean( $id );
-        }
-        when('HASH') {
-            $DATASTORE->stow( $id,'HASH',$data );
-            __clean( $id );
-        }
-        when('Yote::Array') {
-            if( __is_dirty( $id ) ) {
-                $DATASTORE->stow( $id,'ARRAY',$data );
-                __clean( $id );
-            }
-            for my $child (@$data) {
-                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$child} );
-                }
-            }
-        }
-        when('Yote::Hash') {
-            if( __is_dirty( $id ) ) {
-                $DATASTORE->stow( $id, 'HASH', $data );
-            }
-            __clean( $id );
-            for my $child (values %$data) {
-                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$child} );
-                }
-            }
-        }
-        default {
-            if( __is_dirty( $id ) ) {
-                $DATASTORE->stow( $id, $class, $data );
-                __clean( $id );
-            }
-            for my $val (values %$data) {
-                if( $val > 0 && $Yote::ObjProvider::DIRTY->{$val} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$val} );
-                }
-            }
-        }
-    } #given
-
+    if( $class eq 'ARRAY' ) {
+	$DATASTORE->stow( $id,'ARRAY', $data );
+	__clean( $id );
+    }
+    elsif( $class eq 'HASH' ) {
+	$DATASTORE->stow( $id,'HASH',$data );
+	__clean( $id );
+    }
+    elsif( $class eq 'Yote::Array' ) {
+	if( __is_dirty( $id ) ) {
+	    $DATASTORE->stow( $id,'ARRAY',$data );
+	    __clean( $id );
+	}
+	for my $child (@$data) {
+	    if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
+		stow( $Yote::ObjProvider::DIRTY->{$child} );
+	    }
+	}
+    }
+    elsif( $class eq 'Yote::Hash' ) {
+	if( __is_dirty( $id ) ) {
+	    $DATASTORE->stow( $id, 'HASH', $data );
+	}
+	__clean( $id );
+	for my $child (values %$data) {
+	    if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
+		stow( $Yote::ObjProvider::DIRTY->{$child} );
+	    }
+	}
+    }
+    else {
+	if( __is_dirty( $id ) ) {
+	    $DATASTORE->stow( $id, $class, $data );
+	    __clean( $id );
+	}
+	for my $val (values %$data) {
+	    if( $val > 0 && $Yote::ObjProvider::DIRTY->{$val} ) {
+		stow( $Yote::ObjProvider::DIRTY->{$val} );
+	    }
+	}
+    }
 } #stow
 
 sub stow_all {
@@ -491,33 +479,32 @@ sub __raw_data {
     return unless $class;
     my $id = get_id( $obj );
     die unless $id;
-    given( $class ) {
-        when('ARRAY') {
-            my $tied = tied @$obj;
-            if( $tied ) {
-                return $tied->[1];
-            } else {
-                die;
-            }
-        }
-        when('HASH') {
-            my $tied = tied %$obj;
-            if( $tied ) {
-                return $tied->[1];
-            } else {
-                die;
-            }
-        }
-        when('Yote::Array') {
-            return $obj->[1];
-        }
-        when('Yote::Hash') {
-            return $obj->[1];
-        }
-        default {
-            return $obj->{DATA};
-        }
+    if( $class eq 'ARRAY' ) {
+	my $tied = tied @$obj;
+	if( $tied ) {
+	    return $tied->[1];
+	} else {
+	    die;
+	}
     }
+    elsif( $class eq 'HASH' ) {
+	my $tied = tied %$obj;
+	if( $tied ) {
+	    return $tied->[1];
+	} else {
+	    die;
+	}
+    }
+    elsif( $class eq 'Yote::Array' ) {
+	return $obj->[1];
+    }
+    elsif( $class eq 'Yote::Hash' ) {
+	return $obj->[1];
+    }
+    else {
+	return $obj->{DATA};
+    }
+    
 } #__raw_data
 
 sub __store_weak {
